% Regulation of firing frequency in a computational model of a midbrain
% dopaminergic neuron by Kuznetsova, in J. Comput Neurosci 2010 vol 28, pp
% 389
%
% currents in mA/cm^2; conductance in S/cm^2, time in msec; V in mV; ion
% concentration in mM=(0.001 mol/liter); diameter, length in mu meter;

function dy = ODE_dopamine(t,y)

global I_Napmax K_mNa I_Capmax K_mCa gNa R T F Na_out g_LCa E_Ca tau_Ca g_KV g_KLeak g_NaLeak g_CaLeak
global E_K tau_Kv gKA_d gKA_s gKA_p g_SK K_mSK dp dd g_dp g_pd 
global ds g_ps g_sp Cm fd fp fs fCa Istim

dy=zeros(27,1);

Vd=y(1); Vp=y(2); Vs=y(3); 
Na_in_d=y(4); Na_in_p=y(5); Na_in_s=y(6);
Ca_in_d=y(7); Ca_in_p=y(8); Ca_in_s=y(9);
m_d=y(10); m_p=y(11); m_s=y(12); 
h_d=y(13); h_p=y(14); h_s=y(15);
dL_d=y(16); dL_p=y(17); dL_s=y(18);
n_d=y(19); n_p=y(20); n_s=y(21);
q_d=y(22); q_p=y(23); q_s=y(24);
p_d=y(25); p_p=y(26); p_s=y(27);


%leak currents 
%E_K=-90; E_Ca=120; % mV
%R=8.314; T=308.15; F=96520; 
%Na_out=145; %mM
%g_KLeak=5.5*10^(-6); g_NaLeak=2.375*10^(-6); g_CaLeak=0.0136*10^(-6);  %S/cm^2
E_Na_s=1000*R*T/F*log(Na_out/Na_in_s); %convert E_ion into mV via factor 1000 
E_Na_d=1000*R*T/F*log(Na_out/Na_in_d);     
E_Na_p=1000*R*T/F*log(Na_out/Na_in_p); 

I_NaLeak_s=g_NaLeak*(Vs-E_Na_s);
I_NaLeak_d=g_NaLeak*(Vd-E_Na_d);
I_NaLeak_p=g_NaLeak*(Vp-E_Na_p);

I_KLeak_s=g_KLeak*(Vs-E_K);
I_KLeak_d=g_KLeak*(Vd-E_K);
I_KLeak_p=g_KLeak*(Vp-E_K);

I_CaLeak_s=g_CaLeak*(Vs-E_Ca);
I_CaLeak_d=g_CaLeak*(Vd-E_Ca);
I_CaLeak_p=g_CaLeak*(Vp-E_Ca);

% total leak currents
I_Leak_s=I_CaLeak_s + I_KLeak_s + I_NaLeak_s;
I_Leak_d=I_CaLeak_d + I_KLeak_d + I_NaLeak_d;
I_Leak_p=I_CaLeak_p + I_KLeak_p + I_NaLeak_p;

% % sodium pump currents 
% I_Napmax=0.0092; % in m A/cm^2 
% K_mNa=10; %mM
if Na_in_s==0
    I_Napump_s = 0;
else
    I_Napump_s = I_Napmax/(1+ (K_mNa/Na_in_s)^1.5);
end
if Na_in_d==0
    I_Napump_d = 0;
else
    I_Napump_d = I_Napmax/(1+ (K_mNa/Na_in_d)^1.5);
end
if Na_in_p==0
    I_Napump_p = 0;
else
    I_Napump_p = I_Napmax/(1+ (K_mNa/Na_in_p)^1.5);
end

% % Calcium pump current
% I_Capmax=0.00191; % m A/cm^2 
% K_mCa=0.0005; %mM;
I_Capump_s = I_Capmax/(1+K_mCa/Ca_in_s);
I_Capump_p = I_Capmax/(1+K_mCa/Ca_in_p);
I_Capump_d = I_Capmax/(1+K_mCa/Ca_in_d);

% % Fast sodium current
% gNa=550*10^(-6); % S/cm^2
% R=8.314; T=308.15; F=96520; Na_out=145; %mM
I_Na_s = gNa*m_s^3*h_s*(Vs-E_Na_s);
I_Na_p = gNa*m_p^3*h_p*(Vp-E_Na_p);
I_Na_d = gNa*m_d^3*h_d*(Vd-E_Na_d);

% time constant for fast sodium currents in msec
tau_m_s = 1/(1+exp((Vs+45)/1.5))-1/(1+exp((Vs+65)/0.5))+0.04; 
tau_m_p = 1/(1+exp((Vp+45)/1.5))-1/(1+exp((Vp+65)/0.5))+0.04;
tau_m_d = 1/(1+exp((Vd+45)/1.5))-1/(1+exp((Vd+65)/0.5))+0.04;

tau_h_s = 168/(1+exp((Vs+29)/4.5))-168/(1+exp((Vs+49)/2))+2;
tau_h_p = 168/(1+exp((Vp+29)/4.5))-168/(1+exp((Vp+49)/2))+2;
tau_h_d = 168/(1+exp((Vd+29)/4.5))-168/(1+exp((Vd+49)/2))+2;

% % L-type Calcium current
% g_LCa=11.196*10^(-6); % S/cm^2
% E_Ca=120; %mV
I_LCa_s = g_LCa*dL_s*(Vs-E_Ca);
I_LCa_p = g_LCa*dL_p*(Vp-E_Ca);
I_LCa_d = g_LCa*dL_d*(Vd-E_Ca);

% % time constant for calcium current
% tau_Ca=18; %msec
tau_dL_s = tau_Ca*exp(-((Vs+70)^2)/625)+0.3;
tau_dL_p = tau_Ca*exp(-((Vp+70)^2)/625)+0.3;
tau_dL_d = tau_Ca*exp(-((Vd+70)^2)/625)+0.3;

% % Delayed rectifier current
% g_KV=665*10^(-6); % S/cm^2
% E_K=-90; %mV
I_KV_s = g_KV*n_s^3*(Vs-E_K);
I_KV_p = g_KV*n_p^3*(Vp-E_K);
I_KV_d = g_KV*n_d^3*(Vd-E_K);

% % time constant for Delayed rectifier current
% tau_Kv=19; %msec
tau_n_s = 19.0/(1+exp((Vs+39.0)/8.0))+1;
tau_n_p = 19.0/(1+exp((Vp+39.0)/8.0))+1;
tau_n_d = 19.0/(1+exp((Vd+39.0)/8.0))+1;

% % Transient outward potassium current
% gKA_d=266*10^(-6); gKA_s=570*10^(-6); gKA_p=285*10^(-6); % S/cm^2
I_KA_s = gKA_s*p_s^3*q_s*(Vs-E_K);
I_KA_p = gKA_p*p_p^3*q_p*(Vp-E_K);
I_KA_d = gKA_d*p_d^3*q_d*(Vd-E_K);

% time constant for Transient outward potassium current in msec
tau_p_s = 2*exp(-((Vs+50)^2)/550)+1.1;
tau_p_p = 2*exp(-((Vp+50)^2)/550)+1.1;
tau_p_d = 2*exp(-((Vd+50)^2)/550)+1.1;

% % SK potassium current
% g_SK=59*10^(-6); %in S/cm^2 
% K_mSK=0.00019; %mM
if Ca_in_s==0
    c_inf_s = 0;
else
    c_inf_s = 1/(1+(K_mSK/Ca_in_s)^4);
end
I_SK_s = g_SK*c_inf_s*(Vs-E_K);
if Ca_in_p==0
    c_inf_p = 0;
else
    c_inf_p = 1/(1+(K_mSK/Ca_in_p)^4);
end
I_SK_p = g_SK*c_inf_p*(Vp-E_K);
if Ca_in_d==0
    c_inf_d = 0;
else
    c_inf_d = 1/(1+(K_mSK/Ca_in_d)^4);
end
I_SK_d = g_SK*c_inf_d*(Vd-E_K);

Idp = g_dp*(Vd-Vp);
Ipd = g_pd*(Vp-Vd);

Ips = g_ps*(Vp-Vs);
Isp = g_sp*(Vs-Vp);

% % Istim
% Istim=0.1*Istim/(pi*ds*Ls);

%============= voltage ODEs 
% Cm=1; % muF/cm^2; conductance in S/cm^2
% V in mV, dt in msec, current in mA/cm^2, time in msec, concentration in mM
dy(1:3) = [ -1000*(I_Na_d + I_KA_d + I_KV_d + I_SK_d + I_Napump_d + I_LCa_d + I_Leak_d + Idp)/Cm ; %d Vd /dt
            -1000*(I_Na_p + I_KA_p + I_KV_p + I_SK_p + I_Napump_p + I_LCa_p + I_Leak_p + Ipd + Ips)/Cm ; %d Vp /dt
            -1000*(I_Na_s + I_KA_s + I_KV_s + I_SK_s + I_Napump_s + I_LCa_s + I_Leak_s + Isp - Istim)/Cm ]; %d Vs /dt

%============= sodium concentrations ODEs
% fd=2; fp=2; fs=2; %factor, no units
dy(4:6) = [ -4*10^4*fd*(I_Na_d + I_NaLeak_d + 3*I_Napump_d)/(dd*F); %d Na_in_d /dt
            -4*10^4*fp*(I_Na_p + I_NaLeak_p + 3*I_Napump_p)/(dp*F); %d Na_in_p /dt
            -4*10^4*fs*(I_Na_s + I_NaLeak_s + 3*I_Napump_s)/(ds*F)]; %d Na_in_s /dt

%============= calcium concentrations ODEs
% fCa=0.05; %factor, no units
dy(7:9) = [ -2*10^4*fCa*(I_CaLeak_d + I_Capump_d + I_LCa_d)/(dd*F); %d Ca_in_d /dt
            -2*10^4*fCa*(I_CaLeak_p + I_Capump_p + I_LCa_p)/(dp*F);
            -2*10^4*fCa*(I_CaLeak_s + I_Capump_s + I_LCa_s)/(ds*F)];

%============= fast sodium gating variables
% Vhalf_m=34.6; %mV
dy(10:12) = [ (1/(1+exp(-(Vd+34.6)/6))-m_d)/tau_m_d;  %d m_d /dt
              (1/(1+exp(-(Vp+34.6)/6))-m_p)/tau_m_p;
              (1/(1+exp(-(Vs+34.6)/6))-m_s)/tau_m_s];

% Vhalf_h=56.8; %mV
dy(13:15) = [ (1/(1+exp((Vd+56.8)/7.8))-h_d)/tau_h_d; % d h_d /dt
              (1/(1+exp((Vp+56.8)/7.8))-h_p)/tau_h_p;
              (1/(1+exp((Vs+56.8)/7.8))-h_s)/tau_h_s];

%============== calcium gating variables
dy(16:18) = [ (1/(1+exp(-(Vd+45)/5))-dL_d)/tau_dL_d; % d dL_d /dt
              (1/(1+exp(-(Vp+45)/5))-dL_p)/tau_dL_p;
              (1/(1+exp(-(Vs+45)/5))-dL_s)/tau_dL_s];

%============== delayed rectifier current gating variables
dy(19:21) = [ (1/(1+exp(-(Vd+25)/12))-n_d)/tau_n_d; % d n_d /dt
              (1/(1+exp(-(Vp+25)/12))-n_p)/tau_n_p;
              (1/(1+exp(-(Vs+25)/12))-n_s)/tau_n_s];

%============== transient outward potassium current gating variables
% Vhalf_q_d=60; Sq_d=5; Vhalf_q_p=60; Sq_p=5; Vhalf_q_s=56; Sq_s=8; %mV
% tau_q=20; %msec
dy(22:24) = [ (1/(1+exp((Vd+60)/5))-q_d)/20;  %d q_d /dt
              (1/(1+exp((Vp+60)/5))-q_p)/20;
              (1/(1+exp((Vs+56)/8))-q_s)/20];

dy(25:27) = [ (1/(1+exp(-(Vd+43)/24))-p_d)/tau_p_d;  % d p_d /dt
              (1/(1+exp(-(Vp+43)/24))-p_p)/tau_p_p;
              (1/(1+exp(-(Vs+43)/24))-p_s)/tau_p_s];

    


